library benchpress.src.metric.multi_metric;

import 'package:angular2/di.dart' show bind, Injector, OpaqueToken;
import 'package:angular2/src/facade/collection.dart'
    show List, ListWrapper, StringMapWrapper, Map;
import 'package:angular2/src/facade/async.dart' show Future, PromiseWrapper;
import '../metric.dart' show Metric;

class MultiMetric extends Metric {
  static createBindings(childTokens) {
    return [
      bind(_CHILDREN).toAsyncFactory((injector) {
        return PromiseWrapper.all(ListWrapper.map(childTokens, (token) {
          return injector.asyncGet(token);
        }));
      }, [Injector]),
      bind(MultiMetric).toFactory((children) {
        return new MultiMetric(children);
      }, [_CHILDREN])
    ];
  }
  List _metrics;
  MultiMetric(metrics) : super() {
    this._metrics = metrics;
  }
  Future beginMeasure() {
    return PromiseWrapper.all(ListWrapper.map(this._metrics, (metric) {
      return metric.beginMeasure();
    }));
  }
  Future<Map> endMeasure(bool restart) {
    return PromiseWrapper.all(ListWrapper.map(this._metrics, (metric) {
      return metric.endMeasure(restart);
    })).then((values) {
      return mergeStringMaps(values);
    });
  }
  Map describe() {
    return mergeStringMaps(this._metrics.map((metric) {
      return metric.describe();
    }));
  }
}
mergeStringMaps(maps) {
  var result = {};
  ListWrapper.forEach(maps, (map) {
    StringMapWrapper.forEach(map, (value, prop) {
      result[prop] = value;
    });
  });
  return result;
}
var _CHILDREN = new OpaqueToken('MultiMetric.children');
